﻿
local mod = klhtm
local me = { }
mod.netin = me

me.myevents = { "CHAT_MSG_CHANNEL", "CHAT_MSG_SYSTEM" }

-- Special onevent function from Core.lua
me.onevent = function()

	if event == "CHAT_MSG_CHANNEL" then
		
		-- ignore messages outside our channel
		if arg8 ~= mod.net.channelnumber then
			return 
		end
		
		-- ignore non-KLHTM messages
		if string.sub(arg1, 1, 6) ~= "KLHTM " then
			me.logmessage("nonklhtm", string.len(arg1))
			return 
		end
		
		-- ignore messages from people outside the raid group
		if mod.net.isplayeringroup(arg2) == nil then
			me.logmessage("nonklhtm", string.len(arg1))
			return 
		end
		
		-- remove " ...hic!" suffix, and localisations
		local start
		local finish
		start, finish, body = string.find(arg1, "(.-) ?%.%.%..+")
		
		if finish == string.len(arg1) then
			mod.out.printtrace("drunk", "Trimming <drunk> from this message: " .. arg1)
			arg1 = body
		end
		
		-- to get here, it's fine
		me.messagein(arg2, arg1, 7)
	
	elseif event == "CHAT_MSG_SYSTEM" then
					
		if string.find(arg1, string.format(MARKED_AFK_MESSAGE, "")) then
			mod.my.setstate("afk", true)
				
		elseif string.find(arg1, CLEARED_AFK) then
			mod.my.setstate("afk", false)
		end
		
	else
		
		return
	end

end

--[[ 
me.messagein(author, message, startindex)
Processes a message from the chat channel.
<author> is the name of the player who sent the message.
<message> is the original string sent.
<startindex> is the 1-based string index where the message starts.
]]
me.messagein = function(author, message, startindex)
	
	-- get first bit and rest
	local command
	local data
	
	_, _, command, data = string.find(message, "(%a+)(.*)", startindex)
	
	if (command == nil) or (me.commands[command] == nil) then
		-- log as invalid
		me.logmessage("invalid", string.len(message))
		mod.out.printtrace("badmessage", "Received this invalid message: " .. message)
		
	else
		if me.commands[command](author, message, startindex + string.len(command)) then
			-- log as good
			me.logmessage("good", string.len(message))
		else
			-- log as invalid
			me.logmessage("invalid", string.len(message))
			mod.out.printtrace("badmessage", "Received this invalid message: " .. message)
		end
	end
	
end

--[[ 
me.officercheck(author, message)
Checks if the author of an officer-only command is an officer. Otherwise, prints an error to trace.
<author> is the name of the player who sent the message.
<message> is the complete text of the message.
Returns: non-nil iff <author> is an officer.
]]
me.officercheck = function(author, message)
	
	if mod.net.isplayerofficer(author) then
		return true
	
	else
		mod.out.printtrace("badmessage", string.format("|cffffff00%s|r is not an officer, but sent the message |cffffff00%s|r.", author, message))
		return nil
	end

end

--[[
each function is called with 1) author, 2) message, 3) index of first character after the command (probably a space)
they should return non-nil if the message was good, nil otherwise
]]
me.commands = 
{
	-- clearing the raid threat
	["clear"] = function(author, message)
		
		-- author must be officer
		if me.officercheck(author, message) then
			
			mod.out.print(string.format(mod.string.get("print", "network", "threatreset"), author))
			
			mod.table.resetraidthreat()
			mod.table.clearraidtable()
			
			return true
		end
		
	end,
	
	-- updating your threat value
	["threat"] = function(author, message, start)
	
		local value = tonumber(string.sub(message, start + 1))
		
		-- check for validity
		if value == nil then 
			return
		end
	
		mod.table.updateplayerthreat(author, value)
		KLHTM_RequestRedraw("raid")
		
		return true 
		
	end,
	
	-- setting the master target
	["target"] = function(author, message, start)
	
		-- check player is an officer
		if me.officercheck(author, message) then
			
			local newmt = string.sub(message, start + 1)
			-- check newmt makes sense
			
			if newmt ~= nil and string.len(newmt) > 0 then	
				mod.boss.newmastertarget(author, newmt)
				KLHTM_RequestRedraw("raid")
				
				return true
			end
		end
	end,

	-- clearing the master target
	["cleartarget"] = function(author, message)
		
		-- check player is an officer
		if me.officercheck(author, message) then
			
			mod.boss.clearmastertarget()
			mod.out.print(string.format(mod.string.get("print", "network", "mtclear"), author))
			KLHTM_RequestRedraw("raid")
		
			return true
		end
	end,
	
	-- starting knockback discovery
	["knockbackstart"] = function(author, message)
		
		-- check author is an officer
		if me.officercheck(author, message) == nil then
			return
		end
		
		mod.boss.isknockbackdiscoveryactive = true
		
		if author == UnitName("player") then
			mod.boss.isknockbackinvestigator = true
		end
		
		mod.out.print(string.format(mod.string.get("print", "network", "knockbackstart"), author))
		return true
		
	end,
	
	-- stopping knockback discovery
	["knockbackstop"] = function(author, message)
		
		-- check author is an officer
		if me.officercheck(author, message) == nil then
			return
		end
		
		mod.boss.isknockbackdiscoveryactive = false
		mod.boss.isknockbackinvestigator = false
	
		mod.out.print(string.format(mod.string.get("print", "network", "knockbackstop"), author))
		return true
		
	end,
	
	-- gaining aggro (knockback discovery feature)
	["aggrogain"] = function(author, message, start)
		
		-- the rest of the message should be their current threat
		local value = tonumber(string.sub(message, start + 1))
		if value == nil then
			return
		end
		
		-- only report messages to knockback investigators
		if mod.boss.isknockbackinvestigator == true then
			mod.out.print(string.format(mod.string.get("print", "network", "aggrogain"), author, value), 5)
		end
		
		return true
		
	end,
	
	-- losing aggro (knockback discovery feature)
	["aggroloss"] = function(author, message, start)
		
		-- the rest of the message should be their current threat
		local value = tonumber(string.sub(message, start + 1))
		if value == nil then
			return
		end
		
		-- only report messages to knockback investigators
		if mod.boss.isknockbackinvestigator == true then
			mod.out.print(string.format(mod.string.get("print", "network", "aggroloss"), author, value), 5)
		end
		
		return true
		
	end,
	
	-- reporting a threat reduction attack (knockback discovery feature)
	["knockaway"] = function(author, message, start)
		
		-- the rest of the message should be their current threat
		local value = tonumber(string.sub(message, start + 1))
		if value == nil then
			return
		end
		
		-- only report messages to knockback investigators
		if mod.boss.isknockbackinvestigator == true then
			mod.out.print(string.format(mod.string.get("print", "network", "knockback"), author, value), 5)
		end
		
		return true
		
	end,
	
	-- additional text for a threat reduction attack (knockback discovery feature)
	["knockawaytext"] = function(author, message, start)
		
		local text = string.sub(message, start + 1)
		if text == nil then
			return
		end
		
		-- only say something if we are monitoring knockbacks
		if mod.boss.isknockbackinvestigator == true then
			mod.out.print(string.format(mod.string.get("print", "network", "knockbackstring"), author, text), 5)
		end
		
		return true
		
	end,
	
	-- telling people to upgrade to a new version
	["version"] = function(author, message, start)
		
		-- check author is an officer
		if me.officercheck(author, message) == nil then
			return
		end
		
		-- next argument should be the version number
		local release, revision
		_, _, release, revision = string.find(message, "(%d+)%.?(%d*)", start + 1)
		
		release = tonumber(release)
		revision = tonumber(revision)
		
		if release == nil then
			return
		end
		
		-- previously, only the release was sent as the version code. Therefore if someone sends in this format, 
		-- their version must be old.
		if revision == nil then

			mod.out.print(string.format(mod.string.get("print", "network", "remoteoldversion"), author, release .. ".x", mod.release .. "." .. mod.revision))
		
		else
		
			-- newer versions send their release and revision numbers, in a dot delimited string.			
			if (release < mod.release) or (release == mod.release and revision < mod.revision) then
				-- other guy has an old version
				mod.out.print(string.format(mod.string.get("print", "network", "remoteoldversion"), author, release .. "." .. revision, mod.release .. "." .. mod.revision))
				
			elseif (release == mod.release) and (revision == mod.revision) then
				-- we have the same version; do nothing
				
			else
				-- we have an older version - upgrade!
				mod.out.print(string.format(mod.string.get("print", "network", "upgraderequest"), author, release .. "." .. revision, mod.release .. "." .. mod.revision))
			end
		end
		
		return true
		
	end,
	
	-- asking the raid group what versions they are using
	["versionquery"] = function(author, message)
		
		-- check author is an officer
		if me.officercheck(author, message) == nil then
			return
		end
	
		-- send our version
		mod.net.sendmessage("versionresponse " .. mod.release .. "." .. mod.revision)
		
		return true
		
	end,
	
	-- response to versionquery
	["versionresponse"] = function(author, message, start)
		
		-- next argument should be the version number
		local value = tonumber(string.sub(message, start + 1))
		if value == nil then
			return
		end
		
		mod.net.addversionresponse(author, value)		
		return true
		
	end,
	
	-- setting the agreed value for a threat reducing attack of a certain mob
	["knockbackvalue"] = function(author, message, start)
	
		-- check author is an officer
		if me.officercheck(author, message) == nil then
			return
		end
	
		-- parse
		local value
		local attack
		
		_, _, value, attack = string.find(message, "([^ ]+) (.+)", start)
		
		-- first argument = the knockback value (decimal)
		value = tonumber(value)
		if (value == nil) or (value > 0) or (value < -1) then
			return
		end
		
		-- other arguments = name of attack. check mastertarget exists
		if mod.boss.mastertarget == nil then
			return
		end
		
		-- check attack exists
		if mod.data.knockbacks[attack] == nil then
			return
		end
		
		-- succes
		mod.data.knockbacks[attack][mod.boss.mastertarget] = value
		mod.out.print(string.format(mod.string.get("print", "network", "knockbackvaluechange"), author, mod.boss.mastertarget, attack, value * 100))		
		
		-- 6) do it!
		mod.data.knockbacks[attack][mod.boss.mastertarget] = value
		return true
		
	end,
	
}
	

------------------------------------------------------------------------

---------------------------------
--   lololol Message Logging   --
---------------------------------

me.messagelog = 
{
	["nonklhtm"] = 
	{
		["count"] = 0,
		["bytes"] = 0,
	},
	["invalid"] = 
	{
		["count"] = 0,
		["bytes"] = 0,
	},
	["good"] = 
	{
		["count"] = 0,
		["bytes"] = 0,
	},
	["total"] = 
	{
		["count"] = 0,
		["bytes"] = 0,
	},
}

--[[ 
me.logmessage(category, length)
Record a message from the mod's chat channel.
<category> is one of the keys in me.messagelog.
<length> is the lenght in bytes of the complete message.
]]
me.logmessage = function(category, length) 

	me.messagelog[category].count = me.messagelog[category].count + 1
	me.messagelog[category].bytes = me.messagelog[category].bytes + length

	me.messagelog["total"].count = me.messagelog["total"].count + 1
	me.messagelog["total"].bytes = me.messagelog["total"].bytes + length

end
